using System;
using System.IO;
using System.Text.RegularExpressions;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.CalDav;

namespace WebDAVServer.CalDav.CalDav
{
    /// <summary>
    /// Represents outbox folder.
    /// </summary>
    /// <remarks>
    /// Folders of this type correspond to the following path: /calendars/[user_name]/outbox/.
    /// </remarks>
    public class ScheduleOutboxFolder : DavFolder, IScheduleOutboxFolder
    {
        public static readonly string ScheduleOutboxFolderName = "outbox";

        /// <summary>
        /// Returns outbox folder that corresponds to path.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        /// <param name="path">Encoded path relative to WebDAV root.</param>
        /// <returns>ScheduleOutboxFolder instance or null if not found.</returns>
        public static ScheduleOutboxFolder GetScheduleOutboxFolder(DavContext context, string path)
        {
            string pattern = string.Format("^/?{0}/(?<user_name>[^/]+)/{1}/?",
                                           CalendarsRootFolder.CalendarsRootFolderName, ScheduleOutboxFolderName);
            if (!Regex.IsMatch(path, pattern))
                return null;

            DirectoryInfo folder = new DirectoryInfo(context.MapPath(path));
            if (!folder.Exists)
                return null;

            return new ScheduleOutboxFolder(folder, context, path);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ScheduleOutboxFolder"/> class.
        /// </summary>
        /// <param name="directoryInfo">Instance of <see cref="DirectoryInfo"/> class with information about the folder in file system.</param>
        /// <param name="context">Instance of <see cref="DavContext"/>.</param>
        /// <param name="path">Relative to WebDAV root folder path.</param>
        private ScheduleOutboxFolder(DirectoryInfo directoryInfo, DavContext context, string path)
            : base(directoryInfo, context, path)
        {
        }
    }
}
