using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.CalDav;
using ITHit.WebDAV.Server.CardDav;
using WebDAVServer.CalDav.CalDav;
using WebDAVServer.CalDav.CardDav;


namespace WebDAVServer.CalDav
{
    /// <summary>
    /// Assists in finding folders that contain calendars and addressbooks.
    /// </summary>
    public class Discovery : ICalendarDiscovery , IAddressbookDiscovery
    {
        /// <summary>
        /// Instance of <see cref="DavContext"/>.
        /// </summary>
        private DavContext context;

        public Discovery(DavContext context)
        {
            this.context = context;
        }

        /// <summary>
        /// Returns list of folders that contain calendars owned by this principal.
        /// </summary>
        /// <remarks>This enables calendars discovery owned by current loged-in principal.</remarks>
        public IEnumerable<IItemCollection> CalendarHomeSet
        {
            get
            {
                string calendarsUserFolder = string.Format("{0}/{1}/", CalendarsRootFolder.CalendarsRootFolderName, context.UserName);
                yield return DavFolder.GetFolder(context, calendarsUserFolder);
            }
        }

        /// <summary>
        /// Returns list of folders that contain addressbooks owned by this principal.
        /// </summary>
        /// <remarks>This enables addressbooks discovery owned by current loged-in principal.</remarks>
        public IEnumerable<IItemCollection> AddressbookHomeSet
        {
            get
            {
                string addressbooksUserFolder = string.Format("{0}/{1}/", AddressbooksRootFolder.AddressbooksRootFolderName, context.UserName);
                yield return DavFolder.GetFolder(context, addressbooksUserFolder);
            }
        }
    }
}