
using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.CalDav;
using ITHit.WebDAV.Server.CardDav;

namespace WebDAVServer.CalDav
{
    public class MyCustomHandlerPage : Page
    {

        private Discovery discovery = new Discovery(new DavContext(HttpContext.Current));

        /// <summary>
        /// Gets all user calendars Urls
        /// </summary>
        public IEnumerable<string> AllUserCalendarsUrl
        {
            get 
            {
                List<string> urls = new List<string>();                
                
                // get list of folders that contain user calendars and enumerate calendars in each folder
                foreach (IItemCollection folder in discovery.CalendarHomeSet)
                {
                    IEnumerable<IHierarchyItem> children = folder.GetChildren(new PropertyName[0]);
                    foreach (IHierarchyItem item in children)
                    {
                        if (item is ICalendarFolder)
                        {
                            urls.Add(ApplicationPath + item.Path);
                        }
                    }
                }
                return urls;
            }
        }

        /// <summary>
        /// Gets all user addressbooks Urls
        /// </summary>
        public IEnumerable<string> AllUserAddressbooksUrl
        {
            get
            {
                List<string> urls = new List<string>();

                // get list of folders that contain user addressbooks and enumerate calendars in each folder
                foreach (IItemCollection folder in discovery.AddressbookHomeSet)
                {
                    IEnumerable<IHierarchyItem> children = folder.GetChildren(new PropertyName[0]);
                    foreach (IHierarchyItem item in children)
                    {
                        if (item is IAddressbookFolder)
                        {
                            urls.Add(ApplicationPath + item.Path);
                        }
                    }
                }
                return urls;
            }
        }

        public static string ApplicationPath
        {
            get
            {
                Uri url = HttpContext.Current.Request.Url;
                DavContext context = new DavContext(HttpContext.Current);
                string server = url.Scheme + "://" + url.Host + (url.IsDefaultPort ? "" : ":"+url.Port.ToString()) + "/" + context.Request.ApplicationPath.Trim('/');
                return server.TrimEnd('/') + '/';
            }
        }
    }
}