//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.AddCustomGetHandler)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
//!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
using System.Web;
using System.Web.UI;
//!<# } #>
//!<# if (Params.CalDav || Params.CardDav) { #>
using System.Text.RegularExpressions;
//!<# } #>
using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Class1;
using ITHit.WebDAV.Server.Extensibility;
//!<# if (Params.CalDav) { #>
using ITHit.WebDAV.Server.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDAV.Server.CardDav;
//!<# } #>
//!<# if (false) { #>
// only required to compile project at dev time
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel
{
    //$<IMethodHandler
    /// <summary>
    /// This handler processes GET and HEAD requests to folders returning custom HTML page.
    /// </summary>
    internal class MyCustomGetHandler : IMethodHandler
    {
        /// <summary>
        /// Handler for GET and HEAD request registered with the engine before registering this one.
        /// We call this default handler to handle GET and HEAD for files, because this handler
        /// only handles GET and HEAD for folders.
        /// </summary>
        public IMethodHandler OriginalHandler { get; set; }

        /// <summary>
        /// Gets a value indicating whether output shall be buffered to calculate content length.
        /// Don't buffer output to calculate content length.
        /// </summary>
        public bool EnableOutputBuffering
        {
            get { return false; }
        }

        /// <summary>
        /// Gets a value indicating whether engine shall log response data (even if debug logging is on).
        /// </summary>
        public bool EnableOutputDebugLogging
        {
            get { return false; }
        }

        /// <summary>
        /// Gets a value indicating whether the engine shall log request data.
        /// </summary>
        public bool EnableInputDebugLogging
        {
            get { return false; }
        }

        /// <summary>
        /// Handles GET and HEAD request.
        /// </summary>
        /// <param name="context">Instace of <see cref="DavContextBase"/>.</param>
        /// <param name="item">Instance of <see cref="IHierarchyItem"/> which was returned by
        /// <see cref="DavContextBase.GetHierarchyItem"/> for this request.</param>
        public void ProcessRequest(DavContextBase context, IHierarchyItem item)
        {
            //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
            string exePath = Path.GetDirectoryName(new Uri(Assembly.GetExecutingAssembly().CodeBase).LocalPath);
            string htmlPath = Directory.GetParent(exePath).FullName;
            //!<# } #>
            if (item is IFolder)
            {
                // In case of GET requests to WebDAV folders we serve a web page to display 
                // any information about this server and how to use it.

                // Remember to call EnsureBeforeResponseWasCalled here if your context implementation
                // makes some useful things in BeforeResponse.
                context.EnsureBeforeResponseWasCalled();

                //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
                Page page = (Page)System.Web.Compilation.BuildManager.CreateInstanceFromVirtualPath(
                    "~/MyCustomHandlerPage.aspx", typeof(MyCustomHandlerPage));
                try
                {
                    page.ProcessRequest(HttpContext.Current);
                }
                catch (HttpException)
                {
                    // The remote host closed the connection.
                }
                //!<# } #>

                //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
                using (TextReader reader = new StreamReader(Path.Combine(htmlPath, "MyCustomHandlerPage.html")))
                {
                    string html = reader.ReadToEnd();
                    //!<# if (Params.CalDav || Params.CardDav) { #>
                    html = html.Replace("_webDavServerUrl_", ApplicationPath(context));
                    //!<# } #>
                    //!<# if (Params.CalDav) { #>
                    html = html.Replace("_CALENDARS_", AllUserCalendarsUrl(context));
                    //!<# } #>
                    //!<# if (Params.CardDav) { #>
                    html = html.Replace("_BOOKS_", AllUserAddressbooksUrl(context));
                    //!<# } #>
                    html = html.Replace("_webDavServerRoot_", context.Request.ApplicationPath.TrimEnd('/'));
                    html = html.Replace("_webDavServerVersion_",
                        Assembly.GetAssembly(typeof(DavEngine)).GetName().Version.ToString());

                    context.Response.ContentLength = html.Length;
                    context.Response.ContentType = "text/html; charset=utf-8";
                    if (context.Request.HttpMethod == "GET")
                    {
                        using (var writer = new StreamWriter(context.Response.OutputStream))
                        {
                            writer.Write(html);
                        }
                    }
                }
                //!<# } #>
            }
            //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
            else if (context.Request.RawUrl.StartsWith("/AjaxFileBrowser/"))
            {
                // The "/AjaxFileBrowser/" is not a WebDAV folder. It can be used to store client script files, 
                // images, static HTML files or any other files that does not require access via WebDAV.
                // Any request to the files in this folder will just serve them to client. 

                context.EnsureBeforeResponseWasCalled();
                string filePath = Path.Combine(htmlPath, context.Request.RawUrl.TrimStart('/').Replace("/", "\\"));
                if (!File.Exists(filePath))
                {
                    throw new DavException("File not found: " + filePath, DavStatus.NOT_FOUND);
                }

                using (TextReader reader = new StreamReader(filePath))
                {
                    string html = reader.ReadToEnd();

                    context.Response.ContentLength = html.Length;
                    context.Response.ContentType = "text/html; charset=utf-8";
                    if (context.Request.HttpMethod == "GET")
                    {
                        using (var writer = new StreamWriter(context.Response.OutputStream))
                        {
                            writer.Write(html);
                        }
                    }
                }
            }
            //!<# } #>
            else
            {
                OriginalHandler.ProcessRequest(context, item);
            }
        }
        
        /// <summary>
        /// This handler shall only be invoked for IFolder items or if original handler (which
        /// this handler substitutes) shall be called for the item.
        /// </summary>
        /// <param name="item">Instance of <see cref="IHierarchyItem"/> which was returned by
        /// <see cref="DavContextBase.GetHierarchyItem"/> for this request.</param>
        /// <returns>Returns <c>true</c> if this handler can handler this item.</returns>
        public bool AppliesTo(IHierarchyItem item)
        {
            return item is IFolder || OriginalHandler.AppliesTo(item);
        }

        //!<# if (Params.Hosting == HostingEnvironment.Listener && Params.CalDav) { #>
        /// <summary>
        /// Gets all user calendars Urls
        /// </summary>
        private string AllUserCalendarsUrl(DavContextBase context)
        {
            Discovery discovery = new Discovery(context as DavContext);
            List<string> urls = new List<string>();

            // get list of folders that contain user calendars and enumerate calendars in each folder
            foreach (IItemCollection folder in discovery.CalendarHomeSet)
            {
                IEnumerable<IHierarchyItem> children = folder.GetChildren(new PropertyName[0]);
                foreach (IHierarchyItem item in children)
                {
                    if (item is ICalendarFolder)
                    {
                        urls.Add(ApplicationPath(context) + item.Path);
                    }
                }
            }
            return String.Join("<br>", urls.ToArray());
        }
        //!<# } #>

        //!<# if (Params.Hosting == HostingEnvironment.Listener && Params.CardDav) { #>
        /// <summary>
        /// Gets all user addressbooks Urls
        /// </summary>
        private string AllUserAddressbooksUrl(DavContextBase context)
        {
            Discovery discovery = new Discovery(context as DavContext);
            List<string> urls = new List<string>();

            // get list of folders that contain user addressbooks and enumerate calendars in each folder
            foreach (IItemCollection folder in discovery.AddressbookHomeSet)
            {
                IEnumerable<IHierarchyItem> children = folder.GetChildren(new PropertyName[0]);
                foreach (IHierarchyItem item in children)
                {
                    if (item is IAddressbookFolder)
                    {
                        urls.Add(ApplicationPath(context) + item.Path);
                    }
                }
            }
            return String.Join("<br>", urls.ToArray());
        }
        //!<# } #>

        //!<# if (Params.Hosting == HostingEnvironment.Listener && (Params.CalDav || Params.CardDav)) { #>
        public static string ApplicationPath(DavContextBase context)
        {
            return context.Request.UrlPrefix + context.Request.ApplicationPath;
        }
        //!<# } #>
    }
    //$>
}
