//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.ACL)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
//!<# if ( Params.AuthenticationType == AuthenticationType.Ntlm ) return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Security;
//!<# if (Params.CalDav) { #>
using System.Collections.Specialized;
//!<# } #>

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Acl;
//!<# if (Params.CalDav) { #>
using ITHit.WebDAV.Server.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDAV.Server.CardDav;
//!<# } #>
//!<# if (Params.CalDav) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CalDav;
//!<# } #>
//!<# if (false) { #>
// only required to compile project at dev time
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel.__AclSimpl__.Acl
{
    /// <summary>
    /// Represents user in WebDAV hierarchy.
    /// </summary>
    public class User :
        /*!<# if (Params.CalDav || Params.CardDav) { #>!*/ Discovery/*!<# } #>!*/
        /*!<# if (Params.CalDav || Params.CardDav) { #>!*/ ,/*!<# } #>!*/ IPrincipal
        /*!<# if (Params.CalDav) { #>!*/, ICalendarPrincipal/*!<# } #>!*/
        /*!<# if (Params.CardDav) { #>!*/, IAddressbookPrincipal/*!<# } #>!*/
    {
        private MembershipUser user;

        public User(MembershipUser user, DavContext context) 
            : base(context)
        {
            if (user == null)
            {
                throw new ArgumentNullException("user");
            }

            this.user = user;
        }

        /// <summary>
        /// Creates <see cref="User"/> instance from name.
        /// </summary>
        /// <param name="name">User name.</param>
        /// <returns>Instance of <see cref="User"/> or <c>null</c> if user is not found.</returns>
        public static User FromName(string name, DavContext context)
        {
            return new User(Membership.GetUser(name), context);
        }


        /// <summary>
        /// Gets principal name.
        /// </summary>
        public string Name
        {
            get { return user.UserName; }
        }

        /// <summary>
        /// Gets encoded path to this principal.
        /// </summary>
        public string Path
        {
            get { return UserFolder.PREFIX + "/" + EncodeUtil.EncodeUrlPart(Name); }
        }

        /// <summary>
        /// Gets date when principal was created.
        /// </summary>
        public DateTime Created
        {
            get { return user.CreationDate; }
        }

        /// <summary>
        /// Gets date when principal was modified.
        /// </summary>
        public DateTime Modified
        {
            get { return user.LastPasswordChangedDate; }
        }

        /// <summary>
        /// Creates new user as copy of this one.
        /// </summary>
        /// <param name="destFolder">Is not used as there's no more locations a user can be copied.</param>
        /// <param name="destName">New user name.</param>
        /// <param name="deep">Whether to copy children - is not user.</param>
        /// <param name="multistatus">Is not used as there's no children.</param>
        public void CopyTo(IItemCollection destFolder, string destName, bool deep, MultistatusException multistatus)
        {
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
        }

        /// <summary>
        /// Renames principal.
        /// </summary>
        /// <param name="destFolder">We don't use it as moving groups to different folder is not supported.</param>
        /// <param name="destName">New name.</param>
        /// <param name="multistatus">We don't use it as there're no child objects.</param>
        public void MoveTo(IItemCollection destFolder, string destName, MultistatusException multistatus)
        {
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
        }

        /// <summary>
        /// Deletes the principal.
        /// </summary>
        /// <param name="multistatus">We don't use it currently as there are no child objects.</param>
        public void Delete(MultistatusException multistatus)
        {
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
        }
        
        /// <summary>
        /// Retrieves properties of the user.
        /// </summary>
        /// <param name="props">Properties to retrieve.</param>
        /// <param name="allprop">Whether all properties shall be retrieved.</param>
        /// <returns>Property values.</returns>
        public IEnumerable<PropertyValue> GetProperties(IList<PropertyName> props, bool allprop)
        {
            return new PropertyValue[0];
        }

        /// <summary>
        /// Retrieves names of dead properties.
        /// </summary>
        /// <returns>Dead propery names.</returns>
        public IEnumerable<PropertyName> GetPropertyNames()
        {
            return new PropertyName[0];                       
        }

        /// <summary>
        /// Updates dead properties.
        /// </summary>
        /// <param name="setProps">Properties to set.</param>
        /// <param name="delProps">Properties to delete.</param>
        /// <param name="multistatus">Here we report problems with properties.</param>
        public void UpdateProperties(IList<PropertyValue> setProps, IList<PropertyName> delProps, MultistatusException multistatus)
        {
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
        }

        /// <summary>
        /// We don't implement it - users doesn't support setting members.
        /// The method is here because from WebDAV perspective there's no difference
        /// between users and groups.
        /// </summary>
        /// <param name="members">Members of the group.</param>
        public void SetGroupMembers(IList<IPrincipal> members)
        {
            throw new DavException("User objects can not contain other users.", DavStatus.CONFLICT);
        }

        /// <summary>
        /// Retrieves principal members. Users have no members, so return empty list.
        /// </summary>
        /// <returns>Principal members.</returns>
        public IEnumerable<IPrincipal> GetGroupMembers()
        {
            return new IPrincipal[0];
        }

        /// <summary>
        /// Gets groups to which this principal belongs.
        /// </summary>
        /// <returns>Enumerable with groups.</returns>
        public IEnumerable<IPrincipal> GetGroupMembership()
        {
            return new IPrincipal[0];
        }

        /// <summary>
        /// Checks whether this user is of well-known type.
        /// </summary>
        /// <param name="wellknownPrincipal">Type to check.</param>
        /// <returns><c>true</c> if the user is of specified well-known type.</returns>
        public bool IsWellKnownPrincipal(WellKnownPrincipal wellknownPrincipal)
        {
            return (wellknownPrincipal == WellKnownPrincipal.Unauthenticated) && (Name == "Anonymous");
        }
        //!<# if (Params.CalDav) { #>

        public string ScheduleOutboxUrl
        {
            get { return string.Format("{0}/{1}/{2}/", CalendarsRootFolder.CalendarsRootFolderName, Name, ScheduleOutboxFolder.ScheduleOutboxFolderName); }
        }

        public string ScheduleInboxUrl
        {
            get { return string.Format("{0}/{1}/{2}/", CalendarsRootFolder.CalendarsRootFolderName, Name, ScheduleInboxFolder.ScheduleInboxFolderName); }
        }

        public IEnumerable<string> CalendarUserAddressSet
        {
            get 
            {
                NameValueCollection emailsSection = (NameValueCollection)System.Configuration.ConfigurationManager.GetSection("emails");
                yield return "mailto:" + emailsSection[Name];
            }
        }
        //!<# } #>
    }
}
