//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.ACL)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
//!<# if ( Params.AuthenticationType == AuthenticationType.Ntlm ) return string.Empty; #>
using System.Collections.Generic;
using System.Linq;
using System.Web.Security;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Acl;
//!<# if (false) { #>
// only required to compile project at dev time
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel.__AclSimpl__.Acl
{
    /// <summary>
    /// Logical folder which contains users.
    /// It has path '/acl/users'
    /// </summary>
    public class UserFolder : LogicalFolder, IPrincipalFolder
    {
        /// <summary>
        /// Path to folder which contains users.
        /// </summary>
        public static string PREFIX = AclFolder.PREFIX + "/users";

        /// <summary>
        /// Gets path of folder which contains users.
        /// </summary>
        public static string PATH
        {
            get { return PREFIX + "/"; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UserFolder"/> class.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/> class.</param>
        public UserFolder(DavContext context) : base(context, "users", PATH)
        {
        }

        /// <summary>
        /// Retrieves users.
        /// </summary>
        /// <param name="properties">List of properties which will be retrieved by the engine later.</param>
        /// <returns>Enumerable with users.</returns>
        public override IEnumerable<IHierarchyItem> GetChildren(IList<PropertyName> properties)
        {
            MembershipUserCollection users = Membership.GetAllUsers();
            foreach (MembershipUser user in users)
            {
                yield return new User(user, Context);
            }
        }

        /// <summary>
        /// We don't support creating folders inside this folder.
        /// </summary>        
        public IPrincipalFolder CreateFolder(string name)
        {
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
        }

        /// <summary>
        /// Creates user.
        /// </summary>
        /// <param name="name">User name.</param>
        /// <returns>Newly created user.</returns>
        public IPrincipal CreatePrincipal(string name)
        {
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
        }

        /// <summary>
        /// Finds users whose properties have certain values.
        /// </summary>
        /// <param name="propValues">Properties and values to look for.</param>
        /// <param name="props">Properties that will be requested by the engine from the returned users.</param>
        /// <returns>Enumerable with users whose properties match.</returns>
        public IEnumerable<IPrincipal> FindPrincipalsByPropertyValues(
            IList<PropertyValue> propValues,
            IList<PropertyName> props)
        {
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
        }

        /// <summary>
        /// Returns list of properties which can be used in <see cref="FindPrincipalsByPropertyValues"/>.
        /// </summary>
        /// <returns></returns>
        public IEnumerable<PropertyDescription> GetPrincipalSearcheableProperties()
        {
            return new PropertyDescription[0];
        }

        /// <summary>
        /// Returns <see cref="IPrincipal"/> for the current user.
        /// </summary>
        /// <param name="props">Properties that will be asked later from the user returned.</param>
        /// <returns>Enumerable with users.</returns>
        public IEnumerable<IPrincipal> GetMatchingPrincipals(IList<PropertyName> props)
        {
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
        }
    }
}
