//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.CalDav)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
using System;
using System.IO;
using System.Text.RegularExpressions;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.CalDav;

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CalDav
{
    /// <summary>
    /// Represents inbox folder.
    /// </summary>
    /// <remarks>
    /// Folders of this type correspond to the following path: /calendars/[user_name]/inbox/.
    /// </remarks>
    public class ScheduleInboxFolder : DavFolder, IScheduleInboxFolder
    {
        public static readonly string ScheduleInboxFolderName = "inbox";

        /// <summary>
        /// Returns inbox folder that corresponds to path.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        /// <param name="path">Encoded path relative to WebDAV root.</param>
        /// <returns>ScheduleInboxFolder instance or null if not found.</returns>
        public static ScheduleInboxFolder GetScheduleInboxFolder(DavContext context, string path)
        {
            string pattern = string.Format("^/?{0}/(?<user_name>[^/]+)/{1}/?",
                                           CalendarsRootFolder.CalendarsRootFolderName, ScheduleInboxFolderName);
            if (!Regex.IsMatch(path, pattern))
                return null;

            DirectoryInfo folder = new DirectoryInfo(context.MapPath(path));
            if (!folder.Exists)
                return null;

            return new ScheduleInboxFolder(folder, context, path);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ScheduleInboxFolder"/> class.
        /// </summary>
        /// <param name="directoryInfo">Instance of <see cref="DirectoryInfo"/> class with information about the folder in file system.</param>
        /// <param name="context">Instance of <see cref="DavContext"/>.</param>
        /// <param name="path">Relative to WebDAV root folder path.</param>
        private ScheduleInboxFolder(DirectoryInfo directoryInfo, DavContext context, string path)
            : base(directoryInfo, context, path)
        {
        }
    }
}
