//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.CardDav)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.IO;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.CalDav;


namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CardDav
{
    /// <summary>
    /// Folder that contains user folders which in turn contain addressbooks.
    /// </summary>
    /// <remarks>
    /// This class represents /addressbooks/ folder. It contains user folders, such as /addressbooks/[user_name]/. 
    /// </remarks>
    public class AddressbooksRootFolder : DavFolder
    {
        public static readonly string AddressbooksRootFolderName = "addressbooks";

        /// <summary>
        /// Returns addressbooks root folder that corresponds to path.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        /// <param name="path">Encoded path relative to WebDAV root.</param>
        /// <returns>AddressbooksRootFolder instance or null if not found.</returns>
        public static AddressbooksRootFolder GetAddressbooksRootFolder(DavContext context, string path)
        {
            if (!path.Equals(AddressbooksRootFolderName, StringComparison.OrdinalIgnoreCase))
                return null;

            DirectoryInfo folder = new DirectoryInfo(context.MapPath(path));
            if (!folder.Exists)
                return null;

            return new AddressbooksRootFolder(folder, context, path);
        }

        private AddressbooksRootFolder(DirectoryInfo directory, DavContext context, string path)
            : base(directory, context, path)
        {
        }


        /* If required you can appy some rules, for example prohibit creating files in this folder

        /// <summary>
        /// Prohibit creating files in this folder.
        /// </summary>
        override public IFile CreateFile(string name)
        {
            throw new DavException("Creating files in this folder is not implemented.", DavStatus.NOT_IMPLEMENTED);
        }

        /// <summary>
        /// Prohibit creating folders via WebDAV in this folder.
        /// </summary>
        /// <remarks>
        /// New user folders are created during first login.
        /// </remarks>
        override public void CreateFolder(string name)
        {
            throw new DavException("Creating sub-folders in this folder is not implemented.", DavStatus.NOT_IMPLEMENTED);
        }
         
        /// <summary>
        /// Prohibit copying this folder.
        /// </summary>        
        override public void CopyTo(IItemCollection destFolder, string destName, bool deep, MultistatusException multistatus)
        {
            throw new DavException("Copying this folder is not allowed.", DavStatus.NOT_ALLOWED);
        }
        */

        /// <summary>
        /// Prohibit moving or renaming this folder
        /// </summary>        
        override public void MoveTo(IItemCollection destFolder, string destName, MultistatusException multistatus)
        {
            throw new DavException("Moving or renaming this folder is not allowed.", DavStatus.NOT_ALLOWED);
        }

        /// <summary>
        /// Prohibit deleting this folder.
        /// </summary>        
        override public void Delete(MultistatusException multistatus)
        {
            throw new DavException("Deleting this folder is not allowed.", DavStatus.NOT_ALLOWED);
        }
    }
}
