﻿//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.CardDav)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.IO;
using System.Text.RegularExpressions;

using ITHit.WebDAV.Card;
using ITHit.WebDAV.Server.CardDav;

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CardDav
{
    /// <summary>
    /// Represents address card in addressbook on a CardDAV server.
    /// </summary>
    public class CardFile : DavFile, ICardFile
    {

        /// <summary>
        /// Returns address card file that corresponds to path.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        /// <param name="path">Encoded path relative to WebDAV root.</param>
        /// <returns>CardFile instance or null if not found.</returns>
        public static CardFile GetCardFile(DavContext context, string path)
        {
            string pattern = string.Format(@"^/?{0}/(?<user_name>[^/]+)/(?<addressbook_name>[^/]+)/(?<file_name>[^/]+\.vcf)$",
                              AddressbooksRootFolder.AddressbooksRootFolderName);
            if (!Regex.IsMatch(path, pattern))
                return null;

            FileInfo file = new FileInfo(context.MapPath(path));
            if (!file.Exists)
                return null;

            return new CardFile(file, context, path);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CardFile"/> class.
        /// </summary>
        /// <param name="file"><see cref="FileInfo"/> for corresponding object in file system.</param>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        /// <param name="path">Encoded path relative to WebDAV root.</param>
        private CardFile(FileInfo file, DavContext context, string path)
            : base(file, context, path)
        {
        }

        public override void Read(Stream output, long startIndex, long count)
        {
            base.Read(output, startIndex, count);
        }

        public override bool Write(Stream content, string contentType, long startIndex, long totalFileSize)
        {
            bool result = base.Write(content, contentType, startIndex, totalFileSize);
            return result;
        }
    }
}